import 'package:flutter/material.dart';
import 'package:walk_2_school/leaderboard_data.dart';
import 'pages/home_page.dart';
import 'pages/login_page.dart';
import 'user_data.dart';
import 'package:walk_2_school/pages/startup_page.dart';

final GlobalKey<NavigatorState> navigatorKey = GlobalKey<NavigatorState>();

void main() {
  runApp(MaterialApp(
    home: const MyApp(),
    navigatorKey: navigatorKey,
  ));
}

class MyApp extends StatefulWidget {
  const MyApp({super.key});

  @override
  _MyAppState createState() => _MyAppState();
}

class _MyAppState extends State<MyApp> {
  @override
  Widget build(BuildContext context) {
    return MaterialApp(
      title: 'Walk 2 School',
      theme: ThemeData(
        primarySwatch: Colors.green,
      ),
      home: FutureBuilder<bool>(
        future: initDataPull(),
        builder: (BuildContext context, AsyncSnapshot<bool> snapshot) {
          if (snapshot.connectionState == ConnectionState.done) {
            if (snapshot.hasData && snapshot.data != null) {
              return snapshot.data! ? HomePage() : const LoginPage();
            } else {
              // Handle the null value, e.g., by showing an error message or a fallback widget
              return const Center(child: Text('Error loading data'));
            }
          }
          return const StartupLoadingPage();
        },
      ),
    );
  }

  Future<bool> initDataPull() async {
    try {
      bool loggedIn = await UserData.updateFromServer();
      print("isUserLoggedIn: $loggedIn");
      if (loggedIn) {
        await LeaderboardData.getLeaderboardData();
      }
      return loggedIn;
    } catch (e, stackTrace) {
      print("Error in initDataPull: $e");
      print("Stack trace: $stackTrace"); // Print the stack trace
      return false; // Return a default value when there is an error
    }
  }
}
