import 'package:flutter/material.dart';
import 'package:walk_2_school/leaderboard_data.dart';
import 'package:walk_2_school/standardized_widgets.dart';

class LeaderboardPage extends StatefulWidget {
  const LeaderboardPage({super.key});

  @override
  _LeaderboardPageState createState() => _LeaderboardPageState();
}

class _LeaderboardPageState extends State<LeaderboardPage> {
  LeaderboardData leaderboardData = LeaderboardData.empty();
  bool _isLoading = false;

  Future<void> _getLeaderboardData([bool invalidateCache = false]) async {
    leaderboardData = await LeaderboardData.getLeaderboardData(
        invalidateCache); // Call your getLeaderboardData function
    setState(() {
      _isLoading = false;
      leaderboardData = leaderboardData;
    });
  }

  @override
  void initState() {
    super.initState();
    setState(() {
      _isLoading = true;
    });
    _getLeaderboardData();
  }

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      appBar: AppBar(
        backgroundColor: const Color(0xFF687A3E),
        centerTitle: true,
        title: StandardizedWidgets().titleText('All Time Leaderboard',
            color: Colors.white, fontSize: 20),
        leading: Padding(
          padding: const EdgeInsets.all(8.0),
          child: GestureDetector(
              onTap: () {
                Navigator.pop(context);
              },
              child: const Icon(Icons.arrow_back)),
        ),
        actions: [
          IconButton(
            icon: const Icon(
              Icons.replay_outlined,
              color: Colors.white,
            ),
            onPressed: () {
              setState(() {
                _isLoading = true;
              });
              _getLeaderboardData(true);
            },
          ),
        ],
      ),
      body: _isLoading
          ? const Center(
              child: CircularProgressIndicator(),
            )
          : leaderboardData.leaderboardData.isEmpty
              ? const Center(
                  child: Text('No Leaderboard Data!'),
                )
              : Padding(
                  padding: const EdgeInsets.symmetric(
                      horizontal: 16.0, vertical: 8.0),
                  child: ListView.builder(
                    itemCount: leaderboardData.leaderboardData.length,
                    itemBuilder: (context, index) {
                      final item = leaderboardData.leaderboardData[index];
                      Widget trophy;
                      if (index == 0) {
                        trophy = const Icon(Icons.emoji_events,
                            color: Color.fromARGB(255, 211, 195, 51),
                            size: 40.0);
                      } else if (index == 1) {
                        trophy = Icon(Icons.emoji_events,
                            color: Colors.grey[400], size: 40.0);
                      } else if (index == 2) {
                        trophy = const Icon(Icons.emoji_events,
                            color: Colors.brown, size: 40.0);
                      } else {
                        trophy = Text(
                          '#${index + 1}',
                          style: const TextStyle(
                              color: Colors.black,
                              fontWeight: FontWeight.bold,
                              fontSize: 20.0),
                        );
                      }

                      return Card(
                        elevation: 4.0,
                        shape: RoundedRectangleBorder(
                          borderRadius: BorderRadius.circular(8.0),
                        ),
                        child: ListTile(
                          leading: trophy,
                          title: Text(
                            item['username'],
                            style: const TextStyle(
                              fontWeight: FontWeight.bold,
                              fontSize: 18.0,
                            ),
                          ),
                          trailing: Text('${item['points']} points'),
                        ),
                      );
                    },
                  ),
                ),
      // bottomNavigationBar: bottomNavBar(context, 'leaderboard'),
    );
  }
}
