import 'package:flutter/material.dart';
import 'package:shared_preferences/shared_preferences.dart';
import 'package:http/http.dart' as http;
import 'package:walk_2_school/leaderboard_data.dart';
import 'package:walk_2_school/main.dart';
import 'package:walk_2_school/misc.dart';
import 'package:walk_2_school/pages/forgot_password_page.dart';
import 'package:walk_2_school/user_data.dart';
import 'dart:convert';
import 'home_page.dart';
import 'package:google_fonts/google_fonts.dart';
import 'package:crypto/crypto.dart';
import 'create_account_page.dart';
import 'package:geolocator/geolocator.dart';
import 'package:permission_handler/permission_handler.dart';
import 'package:walk_2_school/standardized_widgets.dart';

class LoginPage extends StatefulWidget {
  const LoginPage({super.key});

  @override
  _LoginPageState createState() => _LoginPageState();
}

String hashPassword(String password) {
  var bytes = utf8.encode(password);
  var digest = sha256.convert(bytes);
  return digest.toString();
}

class _LoginPageState extends State<LoginPage> {
  final _usernameController = TextEditingController();
  final _passwordController = TextEditingController();
  bool _isLoading = false;

  @override
  void initState() {
    super.initState();
    // get permission to use location
    Geolocator.requestPermission();
    Permission.location.isPermanentlyDenied.then((isPermanentlyDenied) {
      if (isPermanentlyDenied) {
        // The user opted to never again see the permission request dialog for this
        // app. The only way to change the permission's status now is to let the
        // user manually enable it in the system settings.
        openAppSettings();
      }
    });
  }

  Future<void> _login() async {
    final username = _usernameController.text;
    final password = _passwordController.text;
    final hashedPassword = hashPassword(password);
    print(hashedPassword);
    final response = await http.post(
      Uri.parse('$backendUrl/authenticate'),
      headers: {
        'Content-Type': 'application/json',
      },
      body: json.encode({
        'username': username,
        'password': hashedPassword,
      }),
    );

    // response body is a JSON string:
    /*
    {
    "message": "Authenticated" or "Invalid credentials"
    "sessionToken": valid session token if authenticated
    }
    */
    if (response.body.contains('Authenticated')) {
      final prefs = await SharedPreferences.getInstance();
      await prefs.setBool('isLoggedIn', true);
      await prefs.setString('username', username);
      print(json.decode(response.body));
      await prefs.setString(
          'sessionToken', json.decode(response.body)['sessionToken']);

      // get user data from backend
      await UserData.updateFromServer();
      await LeaderboardData.getLeaderboardData();
      UserData ud = await UserData.retrieveLocally();
      print("Retrieved user data: $ud");
      Navigator.pushReplacement(navigatorKey.currentContext!,
          MaterialPageRoute(builder: (_) => HomePage()));
    } else {
      ScaffoldMessenger.of(navigatorKey.currentContext!)
          .showSnackBar(const SnackBar(content: Text('Invalid credentials')));
    }
  }

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      // appBar: AppBar(title: Text('Login Page')),

      backgroundColor: StandardizedWidgets.background,
      body: Center(
        child: SingleChildScrollView(
          child: Padding(
            padding: const EdgeInsets.all(16),
            child: Column(
              mainAxisAlignment: MainAxisAlignment.start,
              children: [
                Column(
                  mainAxisAlignment: MainAxisAlignment.center,
                  children: [
                    Image.asset('assets/images/Logo.png'),
                    const SizedBox(
                      height: 35,
                    ),
                    Image.asset('assets/images/Text.png'),
                  ],
                ),
                const SizedBox(
                  height: 45,
                ),
                Row(mainAxisAlignment: MainAxisAlignment.start, children: [
                  Text(
                    'Login',
                    style: GoogleFonts.montserrat(
                      fontSize: 40,
                      fontWeight: FontWeight.w700,
                      color: Colors.black,
                    ),
                  )
                ]),
                const SizedBox(
                  height: 15,
                ),
                StandardizedWidgets().textFields(
                    'Username',
                    StandardizedWidgets().typingFont,
                    _usernameController,
                    false),
                const SizedBox(height: 10),
                StandardizedWidgets().textFields(
                    'Password',
                    StandardizedWidgets().typingFont,
                    _passwordController,
                    true),
                const SizedBox(height: 5),
                Row(
                  mainAxisAlignment: MainAxisAlignment.end,
                  children: [
                    InkWell(
                      onTap: () {
                        Navigator.push(
                          context,
                          MaterialPageRoute(
                              builder: (context) => ForgotPasswordPage()),
                        );
                      },
                      child: Text('Forgot Password?',
                          style: StandardizedWidgets().yellowTextFont),
                    ),
                  ],
                ),
                const SizedBox(height: 30),
                _isLoading
                    ? const CircularProgressIndicator()
                    : ElevatedButton(
                        style: ButtonStyle(
                            backgroundColor: MaterialStateProperty.all<Color>(
                                StandardizedWidgets.green),
                            fixedSize: MaterialStateProperty.all<Size>(
                                const Size(290, 46)),
                            shape: MaterialStateProperty.all<
                                RoundedRectangleBorder>(
                              const RoundedRectangleBorder(
                                  borderRadius:
                                      BorderRadius.all(Radius.circular(20))),
                            )),
                        onPressed: () {
                          setState(() => _isLoading = true);
                          _login()
                              .then((_) => setState(() => _isLoading = false));
                        },
                        child: Text(
                          'Login',
                          style: StandardizedWidgets().loginFont,
                        ),
                      ),
                const SizedBox(height: 28),
                Row(mainAxisAlignment: MainAxisAlignment.center, children: [
                  Text(
                    'Dont have an account?',
                    style: GoogleFonts.montserrat(
                      fontSize: 15,
                      fontWeight: FontWeight.w400,
                      color: Colors.black,
                    ),
                  ),
                  InkWell(
                    onTap: () {
                      Navigator.push(
                        context,
                        MaterialPageRoute(
                            builder: (context) => const CreateAccountPage()),
                      );
                    },
                    child: Text('Register here.',
                        style: StandardizedWidgets().yellowTextFont),
                  ),
                ])
              ],
            ),
          ),
        ),
      ),
    );
  }
}
