import 'package:flutter/material.dart';
import 'dart:async';
//import 'package:fl_chart/fl_chart.dart';
import 'package:shared_preferences/shared_preferences.dart';
import 'package:walk_2_school/main.dart';
import 'package:walk_2_school/pages/redeemed_rewards_page.dart';
import 'package:walk_2_school/reward_data.dart';
import 'package:walk_2_school/standardized_widgets.dart';
import 'package:walk_2_school/misc.dart';
import 'package:walk_2_school/user_data.dart';
import 'package:walk_2_school/bottom_nav_bar.dart';
import 'package:http/http.dart' as http;
import 'dart:convert';

class RewardsPage extends StatefulWidget {
  const RewardsPage({super.key});

  @override
  State<StatefulWidget> createState() => _RewardsPageState();
}

class _RewardsPageState extends State<RewardsPage> {
  UserData _currUserData = UserData.empty();
  RewardData rewardData = RewardData.empty();
  bool _isLoadingRewardData = false;

  Future<void> _getRewardData([bool invalidateCache = true]) async {
    rewardData = await RewardData.getRewardData(
        invalidateCache); // Call your getLeaderboardData function
    setState(() {
      _isLoadingRewardData = false;
      rewardData = rewardData;
    });
  }

  @override
  void initState() {
    super.initState();
    _loadUserData();
    setState(() {
      _isLoadingRewardData = true;
    });
    _getRewardData();
  }

  void _loadUserData() async {
    _currUserData = await UserData.retrieveLocally();
    _currUserData.checkStreakReset();
    setState(() {
      _currUserData = _currUserData;
    });
  }

  @override
  Widget build(BuildContext context) {
    double width = MediaQuery.of(context).size.width;
    //List<FlSpot> list = [];
    return Scaffold(
      appBar: AppBar(
        backgroundColor: const Color(0xFF687A3E),
        centerTitle: true,
        title: StandardizedWidgets()
            .titleText('Available Rewards!', color: Colors.white, fontSize: 20),
        actions: [
          IconButton(
            onPressed: () {
              // Reload the page
              Navigator.pushReplacement(
                context,
                MaterialPageRoute(
                  builder: (BuildContext context) => const RewardsPage(),
                ),
              );
            },
            icon: const Icon(Icons.refresh),
          ),
        ],
      ),
      body: SingleChildScrollView(
        child: Center(
          child: Padding(
            padding: const EdgeInsets.all(8.0),
            child: Column(
              crossAxisAlignment: CrossAxisAlignment.center,
              children: [
                StandardizedWidgets().titleText(
                    'You have ${_currUserData.currPoints} points to spend!',
                    alignment: TextAlign.center,
                    fontSize: 25),
                StandardizedWidgets().spacer(height: 10),
                ElevatedButton(
                  style: ButtonStyle(
                    shape: MaterialStateProperty.all<OutlinedBorder>(
                      RoundedRectangleBorder(
                        borderRadius: BorderRadius.circular(20),
                      ),
                    ),
                    backgroundColor: MaterialStateProperty.all<Color>(
                        StandardizedWidgets.yellow),
                  ),
                  onPressed: () {
                    Navigator.push(
                      context,
                      MaterialPageRoute(
                        builder: (context) => RedeemedRewardsPage(),
                      ),
                    );
                  },
                  child: SizedBox(
                    height: 50,
                    width: width / 2,
                    child: Column(
                      mainAxisAlignment: MainAxisAlignment.center,
                      children: [
                        StandardizedWidgets().subscriptText('Redeemed Rewards',
                            alignment: TextAlign.center, fontSize: 19),
                      ],
                    ),
                  ),
                ),
                StandardizedWidgets().spacer(height: 10),
                _isLoadingRewardData
                    ? Column(
                        crossAxisAlignment: CrossAxisAlignment.center,
                        mainAxisAlignment: MainAxisAlignment.center,
                        children: const [CircularProgressIndicator()],
                      )
                    : Column(
                        children: injectRewardData(rewardData),
                      ),
              ],
            ),
          ),
        ),
      ),
      bottomNavigationBar: bottomNavBar(context, 'rewards', false),
    );
  }

  List<Widget> injectRewardData(RewardData data) {
    List<Widget> rewards = [];
    for (int i = 0; i < data.rewardData.length; i++) {
      rewards.add(GestureDetector(
        child: StandardizedWidgets().rewardCard(
          context,
          data.rewardData.elementAt(i)['name'],
          double.parse(data.rewardData.elementAt(i)['price'].toString()),
          data.rewardData.elementAt(i)['quantity'],
          _currUserData.currPoints.toDouble(),
          data.rewardData.elementAt(i)['url'],
        ),
        onTap: () {
          showDialog(
              context: context,
              builder: (BuildContext context) {
                return AlertDialog(
                  title: StandardizedWidgets()
                      .titleText('Redeem Me!', fontSize: 25),
                  content: StandardizedWidgets().subscriptText(
                      'Do you want to redeem the ${data.rewardData.elementAt(i)['name']}?'),
                  actions: [
                    ElevatedButton(
                      style: ButtonStyle(
                          backgroundColor: MaterialStateProperty.all<Color>(
                              StandardizedWidgets.background)),
                      onPressed: () {
                        Navigator.pop(context);
                      },
                      child: StandardizedWidgets()
                          .subscriptText('Cancel', color: Colors.black),
                    ),
                    ElevatedButton(
                        style: ButtonStyle(
                            backgroundColor: MaterialStateProperty.all<Color>(
                                StandardizedWidgets.green)),
                        onPressed: () async {
                          SharedPreferences prefs =
                              await SharedPreferences.getInstance();
                          String? sessionToken =
                              prefs.getString('sessionToken');
                          final response = await http.post(
                            Uri.parse('$backendUrl/purchase'),
                            headers: {
                              'Content-Type': 'application/json',
                              'Accept': 'application/json',
                            },
                            body: json.encode({
                              'name': data.rewardData.elementAt(i)['name'],
                              'sessionToken': sessionToken
                            }),
                          );
                          Navigator.pop(navigatorKey.currentContext!);

                          if (response.statusCode == 200) {
                            // resync the user data
                            await UserData.updateFromServer();
                            _currUserData.updatePointsAfterPurchase();
                            _loadUserData();
                            showDialog(
                              context: navigatorKey.currentContext!,
                              builder: (context) {
                                return AlertDialog(
                                  title: StandardizedWidgets()
                                      .titleText('Success!', fontSize: 25),
                                  content: StandardizedWidgets().subscriptText(
                                      'You have successfully redeemed the ${data.rewardData.elementAt(i)['name']}!'),
                                  actions: [
                                    ElevatedButton(
                                      style: ButtonStyle(
                                          backgroundColor:
                                              MaterialStateProperty.all<Color>(
                                                  StandardizedWidgets.green)),
                                      onPressed: () {
                                        Navigator.pop(context);
                                      },
                                      child: StandardizedWidgets()
                                          .subscriptText('OK'),
                                    ),
                                  ],
                                );
                              },
                            );
                          } else {
                            showDialog(
                              context: navigatorKey.currentContext!,
                              builder: (context) {
                                // Parse the error message from the JSON response
                                Map<String, dynamic> jsonResponse =
                                    json.decode(response.body);
                                String errorMessage = jsonResponse['message'];

                                return AlertDialog(
                                  title: StandardizedWidgets()
                                      .titleText('Error!', fontSize: 25),
                                  content: StandardizedWidgets().subscriptText(
                                      'Something went wrong. Please try again later.'
                                      '\n\nError: $errorMessage'),
                                  actions: [
                                    ElevatedButton(
                                      style: ButtonStyle(
                                          backgroundColor:
                                              MaterialStateProperty.all<Color>(
                                                  StandardizedWidgets.red)),
                                      onPressed: () {
                                        Navigator.pop(context);
                                      },
                                      child: StandardizedWidgets()
                                          .subscriptText('OK'),
                                    ),
                                  ],
                                );
                              },
                            );
                          }
                        },
                        child: StandardizedWidgets().subscriptText('Redeem!'))
                  ],
                );
              });
        },
      ));
    }
    return rewards;
  }
}
