import 'package:flutter/material.dart';

import 'dart:math';
import 'package:fl_chart/fl_chart.dart';

import 'package:shared_preferences/shared_preferences.dart';
import 'package:walk_2_school/standardized_widgets.dart';
import 'package:walk_2_school/misc.dart';
import 'package:walk_2_school/user_data.dart';
import 'package:walk_2_school/bottom_nav_bar.dart';

import 'package:intl/intl.dart';
import 'package:walk_2_school/data.dart';

class StatisticsPage extends StatefulWidget {
  const StatisticsPage({super.key});

  @override
  State<StatefulWidget> createState() => _StatisticsPageState();
}

class _StatisticsPageState extends State<StatisticsPage> {
  UserData _currUserData = UserData.empty();
  bool _isExpanded = false;
  List<String> timeStamps = [];
  List<String> points = [];
  List<String> distances = [];
  List<String> streaks = [];
  @override
  void initState() {
    super.initState();
    _loadUserData();
  }

  void _loadUserData() async {
    _currUserData = await UserData.retrieveLocally();
    setState(() {
      _currUserData = _currUserData;
    });
    getStatistics();
  }

  List<FlSpot> list = [];
  @override
  Widget build(BuildContext context) {
    double width = MediaQuery.of(context).size.width;
    double height = MediaQuery.of(context).size.height;

    List<Color> pointsGradient = [
      Colors.yellow.shade200,
      Colors.orange,
    ];
    List<Color> distanceGradient = [
      StandardizedWidgets.green,
      Colors.green.shade200,
    ];
    List<Color> streakGradient = [
      StandardizedWidgets.purple,
      Colors.purple.shade200,
    ];
    //final progressProvider = Provider.of<ProgressProvider>(context);
    return Scaffold(
      backgroundColor: StandardizedWidgets.background,
      appBar: AppBar(
        backgroundColor: const Color(0xFF687A3E),
        centerTitle: true,
        title: Text(
          getHelloMessage(),
          style: const TextStyle(
            fontFamily: 'Montserrat',
            fontWeight: FontWeight.bold,
          ),
        ),
      ),
      body: SingleChildScrollView(
        child: Center(
          child: Padding(
            padding: const EdgeInsets.all(8.0),
            child: Column(
              children: [
                Column(
                  children: [
                    // Column(
                    //   children: toText(convertToDayOfWeek(timeStamps)),
                    // ),
                    //StandardizedWidgets().titleText(timeStamp),
                    StandardizedWidgets().generalStyledWrapper(
                      StandardizedWidgets().barChart(
                        pointsGradient,
                        StandardizedWidgets().getTitleData(
                            convPointsToData(
                              timeStamps,
                              points,
                            ),
                            interval: 16000,
                            unit: 'k'),
                        injectData(
                            convPointsToData(
                              timeStamps,
                              points,
                            ),
                            pointsGradient,
                            StandardizedWidgets.yellow),
                      ),
                      createExPanel(
                          intermediateChildren(
                              'points',
                              _currUserData.allTimePoints.toString(),
                              '670',
                              '23',
                              fontSize: 20),
                          width),
                      width: width,
                      height: height / 3,
                      title: StandardizedWidgets()
                          .titleText('Points Over Time', fontSize: 35),
                      titleHeightRatio: 7,
                    ),
                    SingleChildScrollView(
                      scrollDirection: Axis.horizontal,
                      child: Padding(
                        padding: const EdgeInsets.all(8.0),
                        child: Row(
                          children: [
                            StandardizedWidgets().generalStyledWrapper(
                                StandardizedWidgets().barChart(
                                  distanceGradient,
                                  StandardizedWidgets().getTitleData(
                                      convDistancesToData(
                                          timeStamps, distances),
                                      interval: 100,
                                      bottomAngle: 30 * pi / 180,
                                      unit: 'k'),
                                  injectData(
                                    convDistancesToData(timeStamps, distances),
                                    distanceGradient,
                                    StandardizedWidgets.green,
                                  ),
                                ),
                                createExPanel(
                                    intermediateChildren(
                                      'distance',
                                      _currUserData.allTimeDistance.toString(),
                                      '734m',
                                      '16',
                                      fontSize: 12,
                                    ),
                                    width / 2),
                                padding: 15,
                                width: width / 2,
                                height: height / 7,
                                titleHeightRatio: 5,
                                spaceAfterTitle: 15,
                                title: StandardizedWidgets().titleText(
                                    'Distance Over Time',
                                    fontSize: 17)),
                            StandardizedWidgets().spacer(width: 8),
                            StandardizedWidgets().generalStyledWrapper(
                                StandardizedWidgets().barChart(
                                  streakGradient,
                                  StandardizedWidgets().getTitleData(
                                    convStreaksToData(
                                      timeStamps,
                                      streaks,
                                    ),
                                    interval: 100,
                                    bottomAngle: 30 * pi / 180,
                                  ),
                                  injectData(
                                      convStreaksToData(
                                        timeStamps,
                                        streaks,
                                      ),
                                      streakGradient,
                                      StandardizedWidgets.purple),
                                ),
                                createExPanel(
                                    intermediateChildren(
                                      'streaks',
                                      _currUserData.allTimeMaxStreak.toString(),
                                      '5',
                                      '7',
                                      fontSize: 12,
                                    ),
                                    width / 2),
                                padding: 15,
                                width: width / 2,
                                height: height / 7,
                                titleHeightRatio: 5,
                                spaceAfterTitle: 15,
                                title: StandardizedWidgets().titleText(
                                    'Streak Over Time',
                                    fontSize: 20)),
                          ],
                        ),
                      ),
                    ),
                  ],
                ),
              ],
            ),
          ),
        ),
      ),
      bottomNavigationBar: bottomNavBar(context, 'stats', false),
    );
  }

  Widget createExPanel(List<Widget> list, width) {
    return SizedBox(
      //height: 100,
      width: width,
      child: ExpansionPanelList(
        dividerColor: Colors.black,
        expandedHeaderPadding: EdgeInsets.all(0),
        elevation: 0,
        expansionCallback: (int index, bool isExpanded) {
          setState(() {
            _isExpanded = !isExpanded;
          });
        },
        children: [
          ExpansionPanel(
            canTapOnHeader: true,
            headerBuilder: (BuildContext context, bool isExpanded) {
              return ListTile(
                title: StandardizedWidgets().subscriptText('Show More Data'),
              );
            },
            body: Padding(
              padding: const EdgeInsets.all(8.0),
              child: Column(
                crossAxisAlignment: CrossAxisAlignment.start,
                children: list,
              ),
            ),
            isExpanded: _isExpanded,
          ),
        ],
      ),
    );
  }

  List<Widget> intermediateChildren(String totalWhat, String allTimeData,
      String thisWeeksData, String weekDifferential,
      {double fontSize = 15}) {
    List<Widget> text = [];

    text.add(Row(
      children: [
        StandardizedWidgets()
            .subscriptText('Total $totalWhat:', fontSize: fontSize),
        StandardizedWidgets().titleText(' $allTimeData', fontSize: fontSize),
      ],
    ));
    text.add(Row(
      children: [
        //Expanded(
        StandardizedWidgets()
            .subscriptText('This weeks $totalWhat:', fontSize: fontSize),
        //),
        Expanded(
            child: StandardizedWidgets()
                .titleText(' $thisWeeksData', fontSize: fontSize)),
      ],
    ));
    text.add(Row(
      mainAxisAlignment: MainAxisAlignment.start,
      children: [
        const Icon(
          Icons.arrow_circle_up,
          size: 20,
          color: Colors.green,
        ),
        StandardizedWidgets()
            .titleText(' $weekDifferential%', fontSize: fontSize),
        StandardizedWidgets()
            .subscriptText(' from last week', fontSize: fontSize)
      ],
    ));
    return text;
  }

  List<double> generateRandNums(int length, bool sort) {
    List<double> fakedata = [];
    Random nums = Random();
    for (int i = 0; i < length; i++) {
      fakedata.add(nums.nextDouble() * 100);
    }
    if (sort) {
      fakedata.sort();
    }
    return fakedata;
  }

  List<BarChartGroupData> injectData(
      List<Data> xdata, List<Color> gradient, Color borderColor) {
    List<BarChartGroupData> data = [];

    for (int i = 0; i < xdata.length; i++) {
      data.add(BarChartGroupData(
        x: xdata.elementAt(i).id,
        barRods: [
          BarChartRodData(
            gradient: LinearGradient(
              colors: gradient,
              begin: Alignment.bottomCenter,
              end: Alignment.topRight,
            ),
            fromY: 0,
            width: 30,
            borderSide: BorderSide(
              color: borderColor,
              strokeAlign: BorderSide.strokeAlignCenter,
              width: 2,
            ),
            toY: xdata.elementAt(i).y,
            color: xdata.elementAt(i).color,
            borderRadius: const BorderRadius.all(
              Radius.circular(10),
            ),
          )
        ],
      ));
    }
    return data;
  }

  List<Widget> toText(List<String> list) {
    List<Widget> text = [];
    for (int i = 0; i < list.length; i++) {
      text.add(StandardizedWidgets().titleText(list.elementAt(i)));
    }
    return text;
  }

  List<Data> convPointsToData(
    List<String> time,
    List<String> points,
  ) {
    List<Data> data = [];

    for (int i = 0; i < points.length; i++) {
      data.add(
        Data(
          id: i,
          y: int.parse(points.elementAt(i)).toDouble(),
          name: convertToDayOfWeek(time).elementAt(i),
          color: Colors.blue,
        ),
      );
    }

    return data;
  }

  List<Data> convDistancesToData(
    List<String> time,
    List<String> distance,
  ) {
    List<Data> data = [];
    for (int i = 0; i < distance.length; i++) {
      data.add(
        Data(
          id: 1,
          y: double.parse(distance.elementAt(i)),
          name: convertToDayOfWeek(time).elementAt(i),
          color: Colors.blue,
        ),
      );
    }

    return data;
  }

  List<Data> convStreaksToData(
    List<String> time,
    List<String> streak,
  ) {
    List<Data> data = [];

    for (int i = 0; i < streak.length; i++) {
      data.add(
        Data(
          id: 3,
          y: int.parse(streak.elementAt(i)).toDouble(),
          name: convertToDayOfWeek(time).elementAt(i),
          color: Colors.blue,
        ),
      );
    }
    return data;
  }

  List<String> convertToDayOfWeek(List<String> iso8601) {
    List<String> dayOfWeek = [];
    for (int i = 0; i < iso8601.length; i++) {
      dayOfWeek.add(DateFormat('EEEE')
          .format(DateTime.parse(iso8601.elementAt(i)))
          .toString());
    }
    return dayOfWeek;
  }

  void getStatistics() async {
    SharedPreferences prefs = await SharedPreferences.getInstance();
    // String dateTime = prefs.getString('dateTime') ?? '';
    timeStamps = prefs.getStringList('timeStampList') ?? [];
    points = prefs.getStringList('pointsList') ?? [];
    distances = prefs.getStringList('distanceList') ?? [];
    streaks = prefs.getStringList('streakList') ?? [];

    // timeStamp = DateFormat('EEEE').format(DateTime.parse(dateTime));
  }
}
