import 'dart:convert';

import 'package:flutter/material.dart';

import 'package:http/http.dart' as http;
import 'package:walk_2_school/bottom_nav_bar.dart';
import 'package:walk_2_school/current_walkers_data.dart';
import 'package:walk_2_school/misc.dart';
import 'package:walk_2_school/pages/rewards_page.dart';
import 'package:walk_2_school/standardized_widgets.dart';

class WhosWalkingPage extends StatefulWidget {
  const WhosWalkingPage({super.key});

  @override
  _WhosWalkingPageState createState() => _WhosWalkingPageState();
}

class _WhosWalkingPageState extends State<WhosWalkingPage> {
  bool _isLoading = false;
  List<String> listOfWalkers = [];
  CurrentWalkersData currWalkers = CurrentWalkersData.empty();
  @override
  void initState() {
    super.initState();
    setState(() {
      _isLoading = true;
    });
    getWalkerData();
  }

  Future<void> getWalkerData() async {
    currWalkers = await CurrentWalkersData
        .getCurrentWalkersDataFromBackend(); // Call your getLeaderboardData function
    setState(() {
      _isLoading = false;
      currWalkers = currWalkers;
    });
  }

  @override
  Widget build(BuildContext context) {
    double width = MediaQuery.of(context).size.width;
    double height = MediaQuery.of(context).size.height;

    return Scaffold(
      appBar: AppBar(
        backgroundColor: const Color(0xFF687A3E),
        centerTitle: true,
        title: StandardizedWidgets()
            .titleText("Active Users", color: Colors.white, fontSize: 20),
        leading: Padding(
          padding: const EdgeInsets.all(8.0),
          child: GestureDetector(
            onTap: () {
              // TODO: swap to profile Page
            },
            child: GestureDetector(
                onTap: () {
                  Navigator.pop(context);
                },
                child: const Icon(Icons.arrow_back)),
          ),
        ),
        actions: [
          IconButton(
            icon: const Icon(
              Icons.replay_outlined,
              color: Colors.white,
            ),
            onPressed: () {
              setState(() {
                _isLoading = true;
              });
              getWalkerData();
            },
          ),
        ],
      ),
      body: SingleChildScrollView(
        child: Column(
          mainAxisAlignment: MainAxisAlignment.start,
          children: injectCurrentWalkers(currWalkers, width, height),
        ),
      ),
      bottomNavigationBar: bottomNavBar(context, 'stats', false),
    );
  }

  List<Widget> injectCurrentWalkers(
      CurrentWalkersData data, double width, double height) {
    List<Widget> cards = [];

    for (int i = 0; i < data.currentWalkersData.length; i++) {
      cards.add(StandardizedWidgets().currentWalkerCard(
          data.currentWalkersData.elementAt(i).isWalking,
          data.currentWalkersData.elementAt(i).username,
          width,
          height));
    }
    return cards;
  }
}
