import 'dart:math';

import 'package:flutter/material.dart';
import 'package:walk_2_school/standardized_widgets.dart';

class StatCard extends StatefulWidget {
  final IconData icon;
  final String label;
  final String value;
  final String description;
  final Color iconColor;

  const StatCard({
    Key? key,
    required this.icon,
    required this.label,
    required this.value,
    required this.description,
    this.iconColor = Colors.black,
  }) : super(key: key);

  @override
  _StatCardState createState() => _StatCardState();
}

class _StatCardState extends State<StatCard>
    with SingleTickerProviderStateMixin {
  bool _showDescription = false;

  late AnimationController _controller;
  late Animation<double> _frontRotation;
  late Animation<double> _backRotation;

  @override
  void initState() {
    super.initState();

    _controller = AnimationController(
        duration: const Duration(milliseconds: 500), vsync: this);

    _frontRotation = Tween<double>(begin: 0.0, end: pi / 2).animate(
      CurvedAnimation(
        parent: _controller,
        curve: const Interval(0.0, 0.5, curve: Curves.easeIn),
      ),
    );

    _backRotation = Tween<double>(begin: -pi / 2, end: 0.0).animate(
      CurvedAnimation(
        parent: _controller,
        curve: const Interval(0.5, 1.0, curve: Curves.easeOut),
      ),
    );
  }

  @override
  Widget build(BuildContext context) {
    double width = MediaQuery.of(context).size.width;
    double height = MediaQuery.of(context).size.height;
    return SizedBox(
      //height: height / 9,
      width: width / 2.3,
      child: GestureDetector(
        onTap: () {
          if (!_controller.isAnimating) {
            setState(() {
              _showDescription = !_showDescription;
            });

            if (_showDescription) {
              _controller.forward();
            } else {
              _controller.reverse();
            }
          }
        },
        child: AnimatedBuilder(
          animation: _controller,
          builder: (context, child) {
            final isFront = _controller.value < 0.5;

            return Transform(
              alignment: Alignment.center,
              transform: Matrix4.identity()
                ..setEntry(3, 2, 0.001)
                ..rotateY(isFront ? _frontRotation.value : _backRotation.value),
              child: Card(
                shape: RoundedRectangleBorder(
                    borderRadius: BorderRadius.circular(20)),
                elevation: 5,
                child: Padding(
                  padding: const EdgeInsets.all(16),
                  child: isFront
                      ? Row(
                          children: [
                            Icon(widget.icon,
                                size: 40, color: widget.iconColor),
                            const SizedBox(width: 10),
                            FittedBox(
                              fit: BoxFit.scaleDown,
                              alignment: Alignment.topLeft,
                              child: Column(
                                crossAxisAlignment: CrossAxisAlignment.start,
                                mainAxisSize: MainAxisSize.min,
                                children: [
                                  StandardizedWidgets().subscriptText(
                                    widget.label,
                                    fontSize: 15,
                                  ),
                                  SizedBox(
                                    width: MediaQuery.of(context).size.width *
                                        0.2, // Adjust this value as needed
                                    child: StandardizedWidgets().titleText(
                                      widget.value,
                                      fontSize: 20,
                                    ),
                                  ),
                                ],
                              ),
                            ),
                          ],
                        )
                      : Padding(
                          padding: const EdgeInsets.all(8.0),
                          child: StandardizedWidgets()
                              .subscriptText(widget.description),
                        ),
                ),
              ),
            );
          },
        ),
      ),
    );
  }

  @override
  void dispose() {
    _controller.dispose();
    super.dispose();
  }
}
